@@ -8,13 +8,15 @@
 #
 # Make a self signed cert
 
+use File::Copy;
+
 $dir = shift;
-$openssl_path = shift || '/usr/local/ssl/bin';
+$exe_path = shift || '/usr/local/ssl/bin/openssl';
 
 $egd = defined( $ENV{EGD_POOL} ) ?  "-rand $ENV{EGD_POOL}" : '';
 
-open (REQ, "|$openssl_path/openssl req -config $dir/req.conf "
-      . "-x509 -days 36500 -new -keyout $dir/key.pem $egd >$dir/cert.pem")
+open (REQ, "|$exe_path req -config $dir/req.conf "
+      . "-x509 -days 3650 -new -keyout $dir/key.pem $egd >$dir/cert.pem")
     or die "cant open req. check your path ($!)";
 print REQ <<DISTINGUISHED_NAME;
 XX
@@ -28,17 +30,16 @@ sampo\@iki.fi
 DISTINGUISHED_NAME
     ;
 close REQ;
-system "$openssl_path/openssl verify $dir/cert.pem";  # Just to check
+system "$exe_path verify $dir/cert.pem";  # Just to check
 
 # Generate an encrypted password too
-system "$openssl_path/openssl rsa -in $dir/key.pem -des -passout pass:secret -out $dir/key.pem.e"; 
+system "$exe_path rsa -in $dir/key.pem -des -passout pass:secret -out $dir/key.pem.e"; 
 
 ### Prepare examples directory as certificate directory
 
-$hash = `$openssl_path/openssl x509 -inform pem -hash -noout <$dir/cert.pem`;
+$hash = `$exe_path x509 -inform pem -hash -noout <$dir/cert.pem`;
 chomp $hash;
 unlink "$dir/$hash.0";
-symlink "$dir/cert.pem", "$dir/$hash.0"
-    or die "Can't symlink $dir/$hash.0 ($!)";
+copy "$dir/cert.pem", "$dir/$hash.0" or die "Can't symlink $dir/$hash.0 ($!)";
 
 __END__
